@@ -34,7 +34,8 @@ class PaletteFile:
             if s[:1] == b"#":
                 continue
             if len(s) > 100:
-                raise SyntaxError("bad palette file")
+                msg = "bad palette file"
+                raise SyntaxError(msg)
 
             v = [int(x) for x in s.split()]
             try:
